#include "svc_rpc.h"
#include "rpc_node.h"
#include "tuya_ipc_api.h"
#include "tuya_ipc_event.h"
#include "tuya_ipc_log_demo.h"
#include "tuya_ipc_rpc_auto_api.h"

void rpc_ipc_server_tuya_ipc_get_sdk_info(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;

    PR_INFO("SDK Version:%s\r\n", tuya_ipc_get_sdk_info());


    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_string(reply, JRPC_RESULT, "data", tuya_ipc_get_sdk_info());
    
    return;
}

void rpc_ipc_server_tuya_ipc_dp_report(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;
    int dp_id;
    int type;
    void* p_val = NULL;
    int* p_val_int = NULL;
    char* p_val_str = NULL;
    uint16_t size = 0;
    int cnt = 0;

    jrpc_read_int(request, JRPC_PARAMS, "dp_id", &dp_id);
    jrpc_read_int(request, JRPC_PARAMS, "type", &type);
    if (type == 0 || type == 1 || type == 3) {
        jrpc_read_binary(request, JRPC_PARAMS, "p_val", (uint8_t**)&p_val_int, &size);    
        p_val = p_val_int;
    } else if (type == 2) {
        jrpc_read_binary(request, JRPC_PARAMS, "p_val", (uint8_t**)&p_val_str, &size);
        p_val = p_val_str;
    } else {
        PR_INFO("rpc_ipc_server_tuya_ipc_dp_report unsupport type: %d\n", type);
        ret = OPRT_INVALID_PARM;
        goto exit;
    }
    jrpc_read_int(request, JRPC_PARAMS, "cnt", &cnt);

    PR_INFO("dp_id=%d, type=%d\n", dp_id, type);
    ret = tuya_ipc_dp_report(NULL, dp_id, type, p_val, cnt);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_start_event(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int alarms_types = 0;

    jrpc_read_int(request, JRPC_PARAMS, "alarms_types", &alarms_types);

    ret = tuya_ipc_start_event(alarms_types);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_start_event_with_storage(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int alarms_types = 0;
    int storage_type = 0;

    jrpc_read_int(request, JRPC_PARAMS, "alarms_types", &alarms_types);
    jrpc_read_int(request, JRPC_PARAMS, "storage_type", &storage_type);

    ret = tuya_ipc_start_event_with_storage(alarms_types, storage_type);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_stop_event(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int alarms_types = 0;

    jrpc_read_int(request, JRPC_PARAMS, "alarms_types", &alarms_types);

    ret = tuya_ipc_stop_event(alarms_types);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_stop_event_with_storage(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int alarms_types = 0;
    int storage_type = 0;

    jrpc_read_int(request, JRPC_PARAMS, "alarms_types", &alarms_types);
    jrpc_read_int(request, JRPC_PARAMS, "storage_type", &storage_type);

    ret = tuya_ipc_stop_event_with_storage(alarms_types, storage_type);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_notify_motion_detect(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    char *snap_buffer = NULL;
    int snap_size = 0;
    int read_size = 0;
    int type = 0;
    
    jrpc_read_binary(request, JRPC_PARAMS, "snap_buffer", (uint8_t **)&snap_buffer, (uint16_t *)&read_size);
    jrpc_read_int(request, JRPC_PARAMS, "snap_size", &snap_size);
    jrpc_read_int(request, JRPC_PARAMS, "type", &type);

    if (read_size != snap_size) {
        PR_ERR("addr size %d is mismatch size %d\n", read_size, snap_size);
        ret = OPRT_INVALID_PARM;
        goto exit;
    }

    ret = tuya_ipc_notify_motion_detect(snap_buffer, snap_size, type);
    PR_INFO("notification type is:%d upload result is:%d \n", type, ret);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_notify_door_bell_press(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    char *snap_buffer = NULL;
    int snap_size = 0;
    int read_size = 0;
    int type = 0;
    
    jrpc_read_binary(request, JRPC_PARAMS, "snap_buffer", (uint8_t **)&snap_buffer, (uint16_t *)&read_size);
    jrpc_read_int(request, JRPC_PARAMS, "snap_size", &snap_size);
    jrpc_read_int(request, JRPC_PARAMS, "type", &type);

    if (read_size != snap_size) {
        PR_ERR("addr size %d is mismatch size %d\n", read_size, snap_size);
        ret = OPRT_INVALID_PARM;
        goto exit;
    }

    ret = tuya_ipc_notify_door_bell_press(snap_buffer, snap_size, type);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_door_bell_press(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int doorbell_type = 0;
    char *snap_buffer = NULL;
    int snap_size = 0;
    int read_size = 0;
    int type = 0;
    
    jrpc_read_int(request, JRPC_PARAMS, "doorbell_type", &doorbell_type);
    jrpc_read_binary(request, JRPC_PARAMS, "snap_buffer", (uint8_t **)&snap_buffer, (uint16_t *)&read_size);
    jrpc_read_int(request, JRPC_PARAMS, "snap_size", &snap_size);
    jrpc_read_int(request, JRPC_PARAMS, "type", &type);

    if (read_size != snap_size) {
        PR_ERR("addr size %d is mismatch size %d\n", read_size, snap_size);
        ret = OPRT_INVALID_PARM;
        goto exit;
    }

    ret = tuya_ipc_door_bell_press(doorbell_type, snap_buffer, snap_size, type);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_notify_with_event(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    char *snap_buffer = NULL;
    int snap_size = 0;
    int read_size = 0;
    int type = 0;
    int name = 0;
    int is_notify = 0;
    
    jrpc_read_binary(request, JRPC_PARAMS, "snap_buffer", (uint8_t **)&snap_buffer, (uint16_t *)&read_size);
    jrpc_read_int(request, JRPC_PARAMS, "snap_size", &snap_size);
    jrpc_read_int(request, JRPC_PARAMS, "type", &type);
    jrpc_read_int(request, JRPC_PARAMS, "name", &name);

    if (read_size != snap_size) {
        PR_ERR("addr size %d is mismatch size %d\n", read_size, snap_size);
        ret = OPRT_INVALID_PARM;
        goto exit;
    }

    ret = tuya_ipc_notify_with_event(snap_buffer, snap_size, type, name);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_start_storage(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int storage_type = 0;

    jrpc_read_int(request, JRPC_PARAMS, "storage_type", &storage_type);

    ret = tuya_ipc_start_storage(storage_type);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_stop_storage(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int storage_type = 0;

    jrpc_read_int(request, JRPC_PARAMS, "storage_type", &storage_type);

    ret = tuya_ipc_stop_storage(storage_type);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_notify_alarm(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    char *snap_buffer = NULL;
    int snap_size = 0;
    int read_size = 0;
    int name = 0;
    int is_notify = 0;
    
    jrpc_read_binary(request, JRPC_PARAMS, "snap_buffer", (uint8_t **)&snap_buffer, (uint16_t *)&read_size);
    jrpc_read_int(request, JRPC_PARAMS, "snap_size", &snap_size);
    jrpc_read_int(request, JRPC_PARAMS, "name", &name);
    jrpc_read_int(request, JRPC_PARAMS, "is_notify", &is_notify);

    if (read_size != snap_size) {
        PR_ERR("addr size %d is mismatch size %d\n", read_size, snap_size);
        ret = OPRT_INVALID_PARM;
        goto exit;
    }

    ret = tuya_ipc_notify_alarm(snap_buffer, snap_size, name, is_notify);
    PR_INFO("notification name is:%d upload result is:%d \n", name, ret);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_event_module_init(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    int max_event = 0;
    int alarms_of_event = 0;
    int alarm_cnt_per_event = 0;
    TUYA_ALARM_BITMAP_T bitmap[1] = {0};

    jrpc_read_int(request, JRPC_PARAMS, "max_event", &max_event);
    jrpc_read_int(request, JRPC_PARAMS, "alarms_of_event", &alarms_of_event);
    jrpc_read_int(request, JRPC_PARAMS, "alarm_cnt_per_event", &alarm_cnt_per_event);
    if (max_event < 1) {
        PR_ERR("rpc_ipc_server_tuya_ipc_event_module_init param error");
        ret = OPRT_INVALID_PARM;
        goto exit;
    }
    if (max_event >= 1) {
        max_event = 1;
    }

    ret = tuya_ipc_event_add_alarm_types(&bitmap[0], alarms_of_event);
    if (ret != OPRT_OK) {
        PR_ERR("rpc_ipc_server_tuya_ipc_event_module_init add alarm types error: %d\n", ret);
        goto exit;
    }
    ret = tuya_ipc_event_module_init(max_event, bitmap, alarm_cnt_per_event);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void rpc_ipc_server_tuya_ipc_get_register_status(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    IPC_REGISTER_STATUS_E status = tuya_ipc_get_register_status();

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", status);

    return;
}

void rpc_ipc_server_tuya_ipc_get_service_time(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    TIME_T time_utc = 0;
    INT_T time_zone = 0;

    ret = tuya_ipc_get_service_time(&time_utc, &time_zone);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "time_utc", time_utc);
    jrpc_write_int(reply, JRPC_RESULT, "time_zone", time_zone);

    return;
}

void rpc_ipc_server_tuya_ipc_check_in_dls(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    TIME_T time_utc = 0;
    BOOL_T is_dls = 0;

    jrpc_read_int(request, JRPC_PARAMS, "time_utc", &time_utc);

    ret = tuya_ipc_check_in_dls(time_utc, &is_dls);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", is_dls);

    return;
}

void rpc_ipc_server_tuya_ipc_get_local_time(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    TIME_T in_time = 0;
    struct tm local_time = {0};

    jrpc_read_int(request, JRPC_PARAMS, "in_time", &in_time);

    ret = tuya_ipc_get_local_time(in_time, &local_time);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "localTimeTmYear", local_time.tm_year);
    jrpc_write_int(reply, JRPC_RESULT, "localTimeTmMon", local_time.tm_mon);
    jrpc_write_int(reply, JRPC_RESULT, "localTimeTmMday", local_time.tm_mday);
    jrpc_write_int(reply, JRPC_RESULT, "localTimeTmHour", local_time.tm_hour);
    jrpc_write_int(reply, JRPC_RESULT, "localTimeTmMin", local_time.tm_min);
    jrpc_write_int(reply, JRPC_RESULT, "localTimeTmSec", local_time.tm_sec);

    return;
}

void rpc_ipc_server_tuya_ipc_get_utc_tm(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    struct tm utc_time = {0};

    ret = tuya_ipc_get_utc_tm(&utc_time);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "utcTimeTmYear", utc_time.tm_year);
    jrpc_write_int(reply, JRPC_RESULT, "utcTimeTmMon", utc_time.tm_mon);
    jrpc_write_int(reply, JRPC_RESULT, "utcTimeTmMday", utc_time.tm_mday);
    jrpc_write_int(reply, JRPC_RESULT, "utcTimeTmHour", utc_time.tm_hour);
    jrpc_write_int(reply, JRPC_RESULT, "utcTimeTmMin", utc_time.tm_min);
    jrpc_write_int(reply, JRPC_RESULT, "utcTimeTmSec", utc_time.tm_sec);

    return;
}

void rpc_ipc_server_tuya_ipc_get_utc_time(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    TIME_T time_utc = 0;

    ret = tuya_ipc_get_utc_time(&time_utc);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "time_utc", time_utc);

    return;
}

void rpc_ipc_server_tuya_ipc_get_free_ram(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    ULONG_T free = 0;

    ret = tuya_ipc_get_free_ram(&free);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "free", free);

    return;
}

void rpc_ipc_server_tuya_ipc_get_local_key(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    CHAR_T dest_key_buf[256] = {0};
    UINT_T len = sizeof(dest_key_buf);

    ret = tuya_ipc_get_local_key(dest_key_buf, &len);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_string(reply, JRPC_RESULT, "dest_key_buf", dest_key_buf);
    jrpc_write_int(reply, JRPC_RESULT, "len", len);

    return;
}

void rpc_ipc_server_tuya_ipc_get_device_id(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    CHAR_T devid[256] = {0};
    INT_T id_len = sizeof(devid);

    ret = tuya_ipc_get_device_id(devid, &id_len);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_string(reply, JRPC_RESULT, "devid", devid);
    jrpc_write_int(reply, JRPC_RESULT, "id_len", id_len);

    return;
}

void rpc_ipc_server_tuya_ipc_get_low_power_server(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    UINT_T ip = 0;
    UINT_T port = 0;

    ret = tuya_ipc_get_low_power_server(&ip, &port);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "ip", ip);
    jrpc_write_int(reply, JRPC_RESULT, "port", port);

    return;
}

void rpc_ipc_server_tuya_ipc_get_low_power_server_v2(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    CHAR_T domain[256] = {0};
    UINT_T ip = 0;
    UINT_T port = 0;

    ret = tuya_ipc_get_low_power_server_v2(domain, sizeof(domain), &ip, &port);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_string(reply, JRPC_RESULT, "domain", domain);
    jrpc_write_int(reply, JRPC_RESULT, "ip", ip);
    jrpc_write_int(reply, JRPC_RESULT, "port", port);

    return;
}

void rpc_ipc_server_tuya_ipc_get_mqtt_status(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T status = tuya_ipc_get_mqtt_status();

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", status);

    return;
}

void rpc_ipc_server_tuya_ipc_set_service_time(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    OPERATE_RET ret = OPRT_OK;
    TIME_T new_time_utc = 0;

    jrpc_read_int(request, JRPC_PARAMS, "new_time_utc", &new_time_utc);

    ret = tuya_ipc_set_service_time(new_time_utc);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void tuya_ipc_rpc_server_declare_api_ipc()
{
    tuya_ipc_rpc_server_declare("tuya_ipc_get_sdk_info", rpc_ipc_server_tuya_ipc_get_sdk_info, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_dp_report", rpc_ipc_server_tuya_ipc_dp_report, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_start_event", rpc_ipc_server_tuya_ipc_start_event, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_start_event_with_storage", rpc_ipc_server_tuya_ipc_start_event_with_storage, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_stop_event", rpc_ipc_server_tuya_ipc_stop_event, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_stop_event_with_storage", rpc_ipc_server_tuya_ipc_stop_event_with_storage, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_notify_motion_detect", rpc_ipc_server_tuya_ipc_notify_motion_detect, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_notify_door_bell_press", rpc_ipc_server_tuya_ipc_notify_door_bell_press, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_door_bell_press", rpc_ipc_server_tuya_ipc_door_bell_press, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_notify_with_event", rpc_ipc_server_tuya_ipc_notify_with_event, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_start_storage", rpc_ipc_server_tuya_ipc_start_storage, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_stop_storage", rpc_ipc_server_tuya_ipc_stop_storage, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_notify_alarm", rpc_ipc_server_tuya_ipc_notify_alarm, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_event_module_init", rpc_ipc_server_tuya_ipc_event_module_init, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_register_status", rpc_ipc_server_tuya_ipc_get_register_status, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_service_time", rpc_ipc_server_tuya_ipc_get_service_time, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_check_in_dls", rpc_ipc_server_tuya_ipc_check_in_dls, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_local_time", rpc_ipc_server_tuya_ipc_get_local_time, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_utc_tm", rpc_ipc_server_tuya_ipc_get_utc_tm, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_utc_time", rpc_ipc_server_tuya_ipc_get_utc_time, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_free_ram", rpc_ipc_server_tuya_ipc_get_free_ram, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_local_key", rpc_ipc_server_tuya_ipc_get_local_key, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_device_id", rpc_ipc_server_tuya_ipc_get_device_id, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_low_power_server", rpc_ipc_server_tuya_ipc_get_low_power_server, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_low_power_server_v2", rpc_ipc_server_tuya_ipc_get_low_power_server_v2, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_get_mqtt_status", rpc_ipc_server_tuya_ipc_get_mqtt_status, NULL);
    tuya_ipc_rpc_server_declare("tuya_ipc_set_service_time", rpc_ipc_server_tuya_ipc_set_service_time, NULL);

    return;
}