#include "svc_rpc.h"
#include "json_rpc.h"
#include "tuya_ipc_ptz.h"
#include "tuya_ipc_log_demo.h"
#include "tuya_ipc_rpc_auto_api.h"

static void rpc_ipc_server_tuya_ipc_preset_add(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;
    char *id = NULL;
    char *name = NULL;
    S_PRESET_POSITION_T preset_pos = {0};

    jrpc_read_string(request, JRPC_PARAMS, "preset_pos.id", &id);
    if (id) {
        strncpy(preset_pos.id, id, sizeof(preset_pos.id)-1);
    }
    jrpc_read_string(request, JRPC_PARAMS, "preset_pos.name", &name);
    if (name) {
        strncpy(preset_pos.name, name, sizeof(preset_pos.name)-1);
    }
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.mpId", &preset_pos.mpId);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.pan", &preset_pos.ptz.pan);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.tilt", &preset_pos.ptz.tilt);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.zoom", &preset_pos.ptz.zoom);

    ret = tuya_ipc_preset_add(&preset_pos);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

static void rpc_ipc_server_tuya_ipc_preset_del(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;
    char *preset_id = NULL;

    jrpc_read_string(request, JRPC_PARAMS, "preset_id", &preset_id);

    ret = tuya_ipc_preset_del(preset_id);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

static void rpc_ipc_server_tuya_ipc_preset_get(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int i = 0;
    int ret = 0;
    CHAR_T *cjson_str = NULL;
    S_PRESET_CFG_T preset_cfg = {0};

    ret = tuya_ipc_preset_get(&preset_cfg);

    ty_cJSON *root = ty_cJSON_CreateObject();
    if (root == NULL) {
        PR_ERR("create cJSON failed");
        ret = OPRT_CR_CJSON_ERR;
        goto finish;
    }
    ty_cJSON *preset_array = ty_cJSON_CreateArray();
    if (preset_array == NULL) {
        PR_ERR("create cJSON failed");
        ty_cJSON_Delete(root);
        ret = OPRT_CR_CJSON_ERR;
        goto finish;
    }
    ty_cJSON_AddItemToObject(root, "preset_config", preset_array);

    PR_DEBUG("get preset num is: %d\n", preset_cfg.num);
    for (i = 0; i < preset_cfg.num; i++) {
        PR_DEBUG("%d --> mpid: %d, pan: %d, tilt: %d, zoom: %d\n", i, preset_cfg.position[i].mpId, preset_cfg.position[i].ptz.pan,
                        preset_cfg.position[i].ptz.tilt, preset_cfg.position[i].ptz.zoom);
        ty_cJSON *preset_json = ty_cJSON_CreateObject();
        ty_cJSON_AddNumberToObject(preset_json, "mpid", preset_cfg.position[i].mpId);
        ty_cJSON_AddNumberToObject(preset_json, "pan", preset_cfg.position[i].ptz.pan);
        ty_cJSON_AddNumberToObject(preset_json, "tilt", preset_cfg.position[i].ptz.tilt);
        ty_cJSON_AddNumberToObject(preset_json, "zoom", preset_cfg.position[i].ptz.zoom);
        ty_cJSON_AddStringToObject(preset_json, "id", preset_cfg.position[i].id);
        ty_cJSON_AddStringToObject(preset_json, "name", preset_cfg.position[i].name);
        ty_cJSON_AddItemToArray(preset_array, preset_json);
    }

    cjson_str = ty_cJSON_Print(root);
    ty_cJSON_Delete(root);

finish:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (cjson_str) {
        jrpc_write_string(reply, JRPC_RESULT, "preset_config", cjson_str);
        ty_cJSON_FreeBuffer(cjson_str);
    }

    return;
}

static void rpc_ipc_server_tuya_ipc_preset_add_pic(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    char *addr = NULL;
    int size = 0;
    int read_size = 0;
    int ret = 0;

    jrpc_read_binary(request, JRPC_PARAMS, "addr", (uint8_t **)&addr, (uint16_t *)&read_size);
    jrpc_read_int(request, JRPC_PARAMS, "size", &size);
    if (read_size != size) {
        PR_ERR("addr size %d is mismatch size %d\n", read_size, size);
        ret = OPRT_INVALID_PARM;
        goto finish;
    }

    ret = tuya_ipc_preset_add_pic((CHAR_T *)addr, (UINT_T)size);

finish:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

static void rpc_ipc_server_tuya_ipc_preset_common_add(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;
    int index = 0;
    char *devid = NULL;
    char *id = NULL;
    char *name = NULL;
    S_PRESET_POSITION_T preset_pos = {0};

    jrpc_read_string(request, JRPC_PARAMS, "devid", &devid);
    jrpc_read_int(request, JRPC_PARAMS, "index", &index);
    jrpc_read_string(request, JRPC_PARAMS, "preset_pos.id", &id);
    if (id) {
        strncpy(preset_pos.id, id, sizeof(preset_pos.id)-1);
    }
    jrpc_read_string(request, JRPC_PARAMS, "preset_pos.name", &name);
    if (name) {
        strncpy(preset_pos.name, name, sizeof(preset_pos.name)-1);
    }
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.mpId", &preset_pos.mpId);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.pan", &preset_pos.ptz.pan);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.tilt", &preset_pos.ptz.tilt);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.zoom", &preset_pos.ptz.zoom);

    ret = tuya_ipc_preset_common_add(devid, index, &preset_pos);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

static void rpc_ipc_server_tuya_xvr_preset_common_add(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;
    int index = 0;
    char *node_id = NULL;
    char *id = NULL;
    char *name = NULL;
    S_PRESET_POSITION_T preset_pos = {0};

    jrpc_read_string(request, JRPC_PARAMS, "node_id", &node_id);
    jrpc_read_int(request, JRPC_PARAMS, "index", &index);
    jrpc_read_string(request, JRPC_PARAMS, "preset_pos.id", &id);
    if (id) {
        strncpy(preset_pos.id, id, sizeof(preset_pos.id)-1);
    }
    jrpc_read_string(request, JRPC_PARAMS, "preset_pos.name", &name);
    if (name) {
        strncpy(preset_pos.name, name, sizeof(preset_pos.name)-1);
    }
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.mpId", &preset_pos.mpId);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.pan", &preset_pos.ptz.pan);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.tilt", &preset_pos.ptz.tilt);
    jrpc_read_int(request, JRPC_PARAMS, "preset_pos.ptz.zoom", &preset_pos.ptz.zoom);

    ret = tuya_xvr_preset_common_add(node_id, index, &preset_pos);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

static void rpc_ipc_server_tuya_xvr_preset_common_delete(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;
    char *node_id = NULL;
    char *preset_id = NULL;

    jrpc_read_string(request, JRPC_PARAMS, "node_id", &node_id);
    jrpc_read_string(request, JRPC_PARAMS, "preset_id", &preset_id);

    ret = tuya_xvr_preset_common_delete(node_id, preset_id);

    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

static void rpc_ipc_server_tuya_xvr_preset_common_get(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int i = 0;
    int ret = 0;
    char *node_id = NULL;
    CHAR_T *cjson_str = NULL;
    S_PRESET_CFG_T preset_cfg = {0};

    jrpc_read_string(request, JRPC_PARAMS, "node_id", &node_id);

    ret = tuya_xvr_preset_common_get(node_id, &preset_cfg);

    ty_cJSON *root = ty_cJSON_CreateObject();
    if (root == NULL) {
        PR_ERR("create cJSON failed");
        ret = OPRT_CR_CJSON_ERR;
        goto finish;
    }
    ty_cJSON *preset_array = ty_cJSON_CreateArray();
    if (preset_array == NULL) {
        PR_ERR("create cJSON failed");
        ty_cJSON_Delete(root);
        ret = OPRT_CR_CJSON_ERR;
        goto finish;
    }
    ty_cJSON_AddItemToObject(root, "preset_config", preset_array);

    PR_DEBUG("get preset num is: %d\n", preset_cfg.num);
    for (i = 0; i < preset_cfg.num; i++) {
        PR_DEBUG("%d --> mpid: %d, pan: %d, tilt: %d, zoom: %d\n", i, preset_cfg.position[i].mpId, preset_cfg.position[i].ptz.pan,
                        preset_cfg.position[i].ptz.tilt, preset_cfg.position[i].ptz.zoom);
        ty_cJSON *preset_json = ty_cJSON_CreateObject();
        ty_cJSON_AddNumberToObject(preset_json, "mpid", preset_cfg.position[i].mpId);
        ty_cJSON_AddNumberToObject(preset_json, "pan", preset_cfg.position[i].ptz.pan);
        ty_cJSON_AddNumberToObject(preset_json, "tilt", preset_cfg.position[i].ptz.tilt);
        ty_cJSON_AddNumberToObject(preset_json, "zoom", preset_cfg.position[i].ptz.zoom);
        ty_cJSON_AddStringToObject(preset_json, "id", preset_cfg.position[i].id);
        ty_cJSON_AddStringToObject(preset_json, "name", preset_cfg.position[i].name);
        ty_cJSON_AddItemToArray(preset_array, preset_json);
    }

    cjson_str = ty_cJSON_Print(root);
    ty_cJSON_Delete(root);

finish:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (cjson_str) {
        jrpc_write_string(reply, JRPC_RESULT, "preset_config", cjson_str);
        ty_cJSON_FreeBuffer(cjson_str);
    }

    return;
}

static void rpc_ipc_server_tuya_xvr_preset_pic_common_add(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = 0;
    int index = 0;
    int size = 0;
    int read_size = 0;
    char *node_id = NULL;
    char *addr = NULL;

    jrpc_read_int(request, JRPC_PARAMS, "index", &index);
    jrpc_read_string(request, JRPC_PARAMS, "node_id", &node_id);
    jrpc_read_binary(request, JRPC_PARAMS, "addr", (uint8_t **)&addr, (uint16_t *)&read_size);
    jrpc_read_int(request, JRPC_PARAMS, "size", &size);
    if (read_size != size) {
        PR_ERR("addr size %d is mismatch size %d\n", read_size, size);
        ret = OPRT_INVALID_PARM;
        goto finish;
    }

    ret = tuya_xvr_preset_pic_common_add(node_id, index, addr, size);

finish:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

    return;
}

void tuya_ipc_rpc_server_declare_ptz()
{
    tuya_ipc_rpc_server_declare( "tuya_ipc_preset_add", rpc_ipc_server_tuya_ipc_preset_add, NULL);
    tuya_ipc_rpc_server_declare( "tuya_ipc_preset_del", rpc_ipc_server_tuya_ipc_preset_del, NULL);
    tuya_ipc_rpc_server_declare( "tuya_ipc_preset_get", rpc_ipc_server_tuya_ipc_preset_get, NULL);
    tuya_ipc_rpc_server_declare( "tuya_ipc_preset_add_pic", rpc_ipc_server_tuya_ipc_preset_add_pic, NULL);
    tuya_ipc_rpc_server_declare( "tuya_ipc_preset_common_add", rpc_ipc_server_tuya_ipc_preset_common_add, NULL);
    tuya_ipc_rpc_server_declare( "tuya_xvr_preset_common_add", rpc_ipc_server_tuya_xvr_preset_common_add, NULL);
    tuya_ipc_rpc_server_declare( "tuya_xvr_preset_common_delete", rpc_ipc_server_tuya_xvr_preset_common_delete, NULL);
    tuya_ipc_rpc_server_declare( "tuya_xvr_preset_common_get", rpc_ipc_server_tuya_xvr_preset_common_get, NULL);
    tuya_ipc_rpc_server_declare( "tuya_xvr_preset_pic_common_add", rpc_ipc_server_tuya_xvr_preset_pic_common_add, NULL);

    return;
}
