/**
 * @file tuya_ipc_rpc_server_ring_buffer.c
 * @brief  rpc ring_buffer adapter
 * @version 1.0
 * @date 2021-11-25
 *
 * @copyright Copyright 2021-2031 Tuya Inc. All Rights Reserved.
 *
 */

#include <stdio.h> 
#include "svc_rpc.h"
#include "tuya_error_code.h"
#include "tuya_ring_buffer.h"
#include "tuya_ipc_log_demo.h"
#include "tuya_ipc_rpc_auto_api.h"

void rpc_ipc_server_tuya_ipc_ring_buffer_init(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    INT_T channel = 0 ;
    INT_T device = 0 ; 
    INT_T stream = 0 ;
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"device",&device))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"channel",&channel))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"stream",&stream))) {
        PR_DEBUG("stream err \n");
        goto exit ;
    }

    RING_BUFFER_INIT_PARAM_T param = { 0 } ;

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"pparam.bitrate",&param.bitrate))) {
        PR_DEBUG("param.bitrate err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"pparam.fps",&param.fps))) {
        PR_DEBUG("param.fps err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"pparam.max_buffer_seconds",&param.max_buffer_seconds))) {
        PR_DEBUG("param.max_buffer_seconds err \n");
        goto exit ;
    }
    param.request_key_frame_cb = NULL ;
    
    PR_DEBUG("device %d channel:%d stream:%d bitrate:%d fps:%d  max_buffer_seconds:%d\n",device,channel,stream,param.bitrate,param.fps,param.max_buffer_seconds);
    
    if (OPRT_OK != (ret = tuya_ipc_ring_buffer_init(device,channel,stream,&param))) {
        PR_DEBUG("tuya_ipc_ring_buffer_init err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}


void rpc_ipc_server_tuya_ipc_ring_buffer_uninit(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    INT_T channel = 0 ;
    INT_T device = 0 ;
    INT_T stream = 0 ;
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"device",&device))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"channel",&channel))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"stream",&stream))) {
        PR_DEBUG("stream err \n");
        goto exit ;
    }
    
    PR_DEBUG("device %d channel:%d stream:%d \n",device,channel,stream);
    if (OPRT_OK != ( ret = tuya_ipc_ring_buffer_uninit(device,channel,stream))) {
        PR_DEBUG("tuya_ipc_ring_buffer_uninit err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_ring_buffer_set_max_video_afterwards(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    UINT_T afterwards_sec = 0 ;
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"afterwards_sec",&afterwards_sec))) {
        PR_DEBUG("afterwards_sec err \n");
        goto exit ;
    }
    PR_DEBUG("afterwards_sec:%d \n",afterwards_sec);

    if (OPRT_OK != ( ret = tuya_ipc_ring_buffer_set_max_video_afterwards(afterwards_sec))) {
        PR_DEBUG("tuya_ipc_ring_buffer_set_max_video_afterwards err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}


void rpc_ipc_server_tuya_ipc_ring_buffer_get_video_num_skill(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    INT_T device = 0 ;
    INT_T channel = 0 ;
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"device",&device))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"channel",&channel))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    PR_DEBUG("device:%d , channel:%d \n",device ,channel);

    ret = tuya_ipc_ring_buffer_get_video_num_skill(device,channel);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", 0);
    jrpc_write_int(reply, JRPC_RESULT, "num", ret);
}

void rpc_ipc_server_tuya_ipc_ring_buffer_open(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    INT_T device = 0 ;
    INT_T channel = 0 ;
    IPC_STREAM_E stream = 0 ;
    RBUF_OPEN_TYPE_E open_type = 0 ;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;

        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"device",&device))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"channel",&channel))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"stream",(int *)&stream))) {
        PR_DEBUG("stream err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"open_type",(int *)&open_type))) {
        PR_DEBUG("open_type err \n");
        goto exit ;
    }

    if (NULL == ( handle = tuya_ipc_ring_buffer_open(device,channel,stream,open_type))) {
        PR_DEBUG("tuya_ipc_ring_buffer_get_video_num_skill err \n");
        goto exit ;
    }
   PR_DEBUG("device:%d , channel:%d stream:%d open_type:%d handle:%p\n",device ,channel,stream,open_type,handle);
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (handle) {
        if(OPRT_OK != jrpc_write_binary(reply,JRPC_RESULT,"handle",(uint8_t *)&handle,sizeof(RING_BUFFER_USER_HANDLE_T))) {
            PR_ERR("jrpc_write_binary handle err");
        }
    }
}

void rpc_ipc_server_tuya_ipc_ring_buffer_close(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if (tmp_handle == NULL) {
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;
    PR_DEBUG("handle:%p\n",handle);
    if (OPRT_OK != ( ret = tuya_ipc_ring_buffer_close(handle))) {
        PR_DEBUG("tuya_ipc_ring_buffer_close err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

}

void rpc_ipc_server_tuya_ipc_ring_buffer_append_data(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
   INT_T ret = OPRT_OK;
   RING_BUFFER_USER_HANDLE_T handle = NULL ;
   UCHAR_T *addr = NULL ;
   INT_T size = 0 ;
   UINT16_T binary_size = 0;
   MEDIA_FRAME_TYPE_E type = 0 ;
   UINT64_T pts ;
   RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&binary_size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if(tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"size",&size))) {
        PR_DEBUG("size err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"addr",(uint8_t **)&addr,&binary_size))) {
        PR_DEBUG("addr err \n");
        goto exit ;
    }

    PR_DEBUG("addr:%p\n",addr);
    if (addr == NULL) {
        PR_DEBUG("addr err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"type",(int *)&type))) {
        PR_DEBUG("size err \n");
        goto exit ;
    }
    

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"pts",(int *)&pts))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    PR_DEBUG("size:%d,type:%d,pts:%d,handle:%p,addr:%p\n",size ,type,pts,handle,addr);
    
    if (OPRT_OK !=(ret = tuya_ipc_ring_buffer_append_data(handle,addr,size,type,pts))) {
        PR_DEBUG("tuya_ipc_ring_buffer_append_data err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}


void rpc_ipc_server_tuya_ipc_ring_buffer_append_data_with_timestamp(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UCHAR_T *addr = NULL ;
    INT_T size = 0 ;
    UINT16_T binary_size = 0;
    MEDIA_FRAME_TYPE_E type = 0 ;
    UINT64_T pts ;
    UINT64_T timestamp ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&binary_size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"size",&size))) {
        PR_DEBUG("size err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"addr",&addr,&binary_size))) {
        PR_DEBUG("addr err \n");
        goto exit ;
    }

    PR_DEBUG("addr:%p\n",addr);
    if(addr == NULL){
        PR_DEBUG("addr err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"type",(int *)&type))) {
        PR_DEBUG("size err \n");
        goto exit ;
    }
 
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"pts",(int *)&pts))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"timestamp",(int *)&timestamp))) {
        PR_DEBUG("timestamp err \n");
        goto exit ;
    }

    PR_DEBUG("size:%d,type:%d,pts:%llu,timestamp=%llu ,handle:%p,addr:%p\n",size ,type,pts,timestamp,handle,addr);
    
    if (OPRT_OK !=(ret = tuya_ipc_ring_buffer_append_data_with_timestamp(handle,addr,size,type,pts,timestamp))) {
        PR_DEBUG("tuya_ipc_ring_buffer_append_data err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_ring_buffer_append_extra_data(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UCHAR_T *addr = NULL ;
    INT_T size = 0 ;
    UINT16_T binary_size = 0;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&binary_size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if(tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"size",&size))) {
        PR_DEBUG("size err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"addr",&addr,&binary_size))) {
        PR_DEBUG("addr err \n");
        goto exit ;
    }

    PR_DEBUG("addr:%p\n",addr);
    if (addr == NULL){
        PR_DEBUG("addr err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
       
    PR_DEBUG("size:%d ,handle:%p,addr:%p\n",size ,handle,addr);
    
    if (OPRT_OK !=(ret = tuya_ipc_ring_buffer_append_extra_data(handle,addr,size))) {
        PR_DEBUG("tuya_ipc_ring_buffer_append_data err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_ring_buffer_get_frame(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    BOOL_T is_retry ;
    UINT16_T binary_size = 0;
    RING_BUFFER_NODE_T* ring_buffer_node = NULL ;
    UINT16_T size = 0 ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&binary_size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("tmp_handle:%llu\n",*(UINT64_T*)tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("tmp_handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
     handle = *(UINT64_T*)tmp_handle ;
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"is_retry",&is_retry))) {
        PR_DEBUG("size err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p,is_retry:%d\n",handle,is_retry);
    if (NULL == (ring_buffer_node = tuya_ipc_ring_buffer_get_frame(handle,is_retry))) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("tuya_ipc_ring_buffer_get_frame err \n");
        goto exit ;
    }

exit:
    PR_DEBUG("rpc_ipc_server_tuya_ipc_ring_buffer_get_frame start");
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (ring_buffer_node) {
        
        PR_DEBUG("timestamp:%llu size:%u extra_size:%u",ring_buffer_node->timestamp,ring_buffer_node->size,ring_buffer_node->extra_size);
        jrpc_write_int(reply, JRPC_RESULT, "index", ring_buffer_node->index);
        jrpc_write_int(reply, JRPC_RESULT, "type", ring_buffer_node->type);
        if (ring_buffer_node->size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "size", ring_buffer_node->size);
            jrpc_write_binary(reply,JRPC_RESULT,"raw_data",ring_buffer_node->raw_data,ring_buffer_node->size);
        }
        jrpc_write_binary(reply,JRPC_RESULT,"pts",(uint8_t *)&ring_buffer_node->pts,sizeof(ring_buffer_node->pts));
        jrpc_write_binary(reply,JRPC_RESULT,"timestamp",(uint8_t *)&ring_buffer_node->timestamp,sizeof(ring_buffer_node->timestamp));
        jrpc_write_int(reply, JRPC_RESULT, "seq_no", ring_buffer_node->seq_no);
        jrpc_write_int(reply, JRPC_RESULT, "seq_sync", ring_buffer_node->seq_sync);

        if (ring_buffer_node->extra_size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "extra_size", ring_buffer_node->extra_size);
            jrpc_write_binary(reply,JRPC_RESULT,"extra_data",ring_buffer_node->extra_data,ring_buffer_node->extra_size);
        }
    }
    PR_DEBUG("rpc_ipc_server_tuya_ipc_ring_buffer_get_frame end");


}

void rpc_ipc_server_tuya_ipc_ring_buffer_get_av_frame(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T v_handle = NULL ;
    RING_BUFFER_USER_HANDLE_T a_handle = NULL ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;
    BOOL_T is_retry ;
    RING_BUFFER_NODE_T* ring_buffer_node = NULL ;
    UINT16_T size = 0 ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"v_handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("v_handle err \n");
        goto exit ;
    }

    PR_DEBUG("v_handle:%p\n",tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("v_handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    v_handle = *(UINT64_T*)tmp_handle ;
    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"a_handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("a_handle err \n");
        goto exit ;
    }

    PR_DEBUG("a_handle:%p\n",tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("a_handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    
    a_handle = *(UINT64_T*)tmp_handle ;

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"is_retry",&is_retry))) {
        PR_DEBUG("size err \n");
        goto exit ;
    }

    PR_DEBUG("v_handle:%p,a_handle:%p,is_retry:%d\n",v_handle,a_handle,is_retry);
    if (NULL == (ring_buffer_node = tuya_ipc_ring_buffer_get_av_frame(v_handle,a_handle,is_retry))) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("tuya_ipc_ring_buffer_get_av_frame err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (ring_buffer_node) {
        jrpc_write_int(reply, JRPC_RESULT, "index", ring_buffer_node->index);
        jrpc_write_int(reply, JRPC_RESULT, "type", ring_buffer_node->type);
        if (ring_buffer_node->size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "size", ring_buffer_node->size);
            jrpc_write_binary(reply,JRPC_RESULT,"raw_data",ring_buffer_node->raw_data,ring_buffer_node->size);
        }
        jrpc_write_binary(reply,JRPC_RESULT,"pts",(uint8_t *)&ring_buffer_node->pts,sizeof(ring_buffer_node->pts));
        jrpc_write_binary(reply,JRPC_RESULT,"timestamp",(uint8_t *)&ring_buffer_node->timestamp,sizeof(ring_buffer_node->timestamp));
        jrpc_write_int(reply, JRPC_RESULT, "seq_no", ring_buffer_node->seq_no);
        jrpc_write_int(reply, JRPC_RESULT, "seq_sync", ring_buffer_node->seq_sync);

        if (ring_buffer_node->extra_size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "extra_size", ring_buffer_node->extra_size);
            jrpc_write_binary(reply,JRPC_RESULT,"extra_data",ring_buffer_node->extra_data,ring_buffer_node->extra_size);
        }
    }

}

void rpc_ipc_server_tuya_ipc_ring_buffer_find_pre_by_frame(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT_T frame_num;
    RING_BUFFER_NODE_T* ring_buffer_node = NULL ;
    UINT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    
    handle = *(UINT64_T*)tmp_handle ;
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"frame_num",&frame_num))) {
        PR_DEBUG("frame_num err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p,frame_num:%d\n",handle,frame_num);
    if (NULL == (ring_buffer_node = tuya_ipc_ring_buffer_find_pre_by_frame(handle,frame_num))) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("tuya_ipc_ring_buffer_get_av_frame err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (ring_buffer_node) {
        jrpc_write_int(reply, JRPC_RESULT, "index", ring_buffer_node->index);
        jrpc_write_int(reply, JRPC_RESULT, "type", ring_buffer_node->type);
        if (ring_buffer_node->size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "size", ring_buffer_node->size);
            jrpc_write_binary(reply,JRPC_RESULT,"raw_data",ring_buffer_node->raw_data,ring_buffer_node->size);
        }
        jrpc_write_binary(reply,JRPC_RESULT,"pts",(uint8_t *)&ring_buffer_node->pts,sizeof(ring_buffer_node->pts));
        jrpc_write_binary(reply,JRPC_RESULT,"timestamp",(uint8_t *)&ring_buffer_node->timestamp,sizeof(ring_buffer_node->timestamp));
        jrpc_write_int(reply, JRPC_RESULT, "seq_no", ring_buffer_node->seq_no);
        jrpc_write_int(reply, JRPC_RESULT, "seq_sync", ring_buffer_node->seq_sync);

        if (ring_buffer_node->extra_size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "extra_size", ring_buffer_node->extra_size);
            jrpc_write_binary(reply,JRPC_RESULT,"extra_data",ring_buffer_node->extra_data,ring_buffer_node->extra_size);
        }
    }

}

void rpc_ipc_server_tuya_ipc_ring_buffer_get_pre_frame(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT_T frame_num;
    RING_BUFFER_NODE_T* ring_buffer_node = NULL ;
    UINT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    
     handle = *(UINT64_T*)tmp_handle ;
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"frame_num",&frame_num))) {
        PR_DEBUG("frame_num err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p,frame_num:%d\n",handle,frame_num);
    if (NULL == (ring_buffer_node = tuya_ipc_ring_buffer_get_pre_frame(handle,frame_num))) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("tuya_ipc_ring_buffer_get_av_frame err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (ring_buffer_node) {
        jrpc_write_int(reply, JRPC_RESULT, "index", ring_buffer_node->index);
        jrpc_write_int(reply, JRPC_RESULT, "type", ring_buffer_node->type);
        if (ring_buffer_node->size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "size", ring_buffer_node->size);
            jrpc_write_binary(reply,JRPC_RESULT,"raw_data",ring_buffer_node->raw_data,ring_buffer_node->size);
        }
        jrpc_write_binary(reply,JRPC_RESULT,"pts",(uint8_t *)&ring_buffer_node->pts,sizeof(ring_buffer_node->pts));
        jrpc_write_binary(reply,JRPC_RESULT,"timestamp",(uint8_t *)&ring_buffer_node->timestamp,sizeof(ring_buffer_node->timestamp));
        jrpc_write_int(reply, JRPC_RESULT, "seq_no", ring_buffer_node->seq_no);
        jrpc_write_int(reply, JRPC_RESULT, "seq_sync", ring_buffer_node->seq_sync);

        if (ring_buffer_node->extra_size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "extra_size", ring_buffer_node->extra_size);
            jrpc_write_binary(reply,JRPC_RESULT,"extra_data",ring_buffer_node->extra_data,ring_buffer_node->extra_size);
        }
    }

}

void rpc_ipc_server_tuya_ipc_ring_buffer_get_next_target_frame(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT_T seq_no ;
    MEDIA_FRAME_TYPE_E type ;
    RING_BUFFER_NODE_T* ring_buffer_node = NULL ;
    UINT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;
        
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"seq_no",(int *)&seq_no))) {
        PR_DEBUG("seq_no err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"type",(int *)&type))) {
        PR_DEBUG("type err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p,seq_no:%d type:%d\n",handle,seq_no,type);
    if (NULL == (ring_buffer_node = tuya_ipc_ring_buffer_get_next_target_frame(handle,seq_no,type))) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("tuya_ipc_ring_buffer_get_next_target_frame err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    if (ring_buffer_node) {
        jrpc_write_int(reply, JRPC_RESULT, "index", ring_buffer_node->index);
        jrpc_write_int(reply, JRPC_RESULT, "type", ring_buffer_node->type);
        if (ring_buffer_node->size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "size", ring_buffer_node->size);
            jrpc_write_binary(reply,JRPC_RESULT,"raw_data",ring_buffer_node->raw_data,ring_buffer_node->size);
        }
        jrpc_write_binary(reply,JRPC_RESULT,"pts",(uint8_t *)&ring_buffer_node->pts,sizeof(ring_buffer_node->pts));
        jrpc_write_binary(reply,JRPC_RESULT,"timestamp",(uint8_t *)&ring_buffer_node->timestamp,sizeof(ring_buffer_node->timestamp));
        jrpc_write_int(reply, JRPC_RESULT, "seq_no", ring_buffer_node->seq_no);
        jrpc_write_int(reply, JRPC_RESULT, "seq_sync", ring_buffer_node->seq_sync);

        if (ring_buffer_node->extra_size > 0) {
            jrpc_write_int(reply, JRPC_RESULT, "extra_size", ring_buffer_node->extra_size);
            jrpc_write_binary(reply,JRPC_RESULT,"extra_data",ring_buffer_node->extra_data,ring_buffer_node->extra_size);
        }
    }

}

void rpc_ipc_server_tuya_ipc_ring_buffer_clean_user_state(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;
    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if (tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;
    PR_DEBUG("handle:%p\n",handle);
    tuya_ipc_ring_buffer_clean_user_state(handle) ;

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

}

void rpc_ipc_server_tuya_ipc_ring_buffer_clean_user_state_and_buffer(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if(tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;
    PR_DEBUG("handle:%p\n",handle);
    tuya_ipc_ring_buffer_clean_user_state_and_buffer(handle) ;

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

}

void rpc_ipc_server_tuya_ipc_ring_buffer_if_delay(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret =jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if(tmp_handle == NULL){
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;
    PR_DEBUG("handle:%p\n",handle);
    if (FALSE == tuya_ipc_ring_buffer_if_delay(handle) ) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("handle err \n");
        goto exit ;
    }
    
exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

}


void rpc_ipc_server_tuya_ipc_ring_buffer_get_max_buf_secs(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = OPRT_OK;
   INT_T device = 0 ;
   INT_T channel = 0 ;
   IPC_STREAM_E stream ;
   INT_T max_buf_secs = 0 ;
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"device",&device))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"channel",&channel))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"stream",(int *)&stream))) {
        PR_DEBUG("stream err \n");
        goto exit ;
    }

    PR_DEBUG("device:%d , channel:%d stream:%d\n",device ,channel,stream);
    max_buf_secs = tuya_ipc_ring_buffer_get_max_buf_secs(device,channel,stream) ;

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "max_buf_secs", max_buf_secs);
}


void rpc_ipc_server_tuya_ipc_ring_buffer_set_max_frame_size(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = OPRT_OK;
   INT_T device = 0 ;
   INT_T channel = 0 ;
   IPC_STREAM_E stream ;
   INT_T max_buf_secs = 0 ;
   UINT_T max_frame_size = 0 ;
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"device",&device))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"channel",&channel))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"stream",(int *)&stream))) {
        PR_DEBUG("stream err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"max_frame_size",&max_frame_size))) {
        PR_DEBUG("stream err \n");
        goto exit ;
    }

    PR_DEBUG("device:%d , channel:%d stream:%d\n",device ,channel,stream);
    if (OPRT_OK != (ret = tuya_ipc_ring_buffer_set_max_frame_size(device,channel,stream,max_frame_size))) {
        PR_DEBUG("tuya_ipc_ring_buffer_set_max_frame_size err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void rpc_ipc_server_tuya_ipc_ring_buffer_get_max_frame_size(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    int ret = OPRT_OK;
    INT_T device = 0 ;
    INT_T channel = 0 ;
    IPC_STREAM_E stream ;
    INT_T max_frame_size = 0 ;
    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"device",&device))) {
        PR_DEBUG("device err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"channel",&channel))) {
        PR_DEBUG("channel err \n");
        goto exit ;
    }

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"stream",(int *)&stream))) {
        PR_DEBUG("stream err \n");
        goto exit ;
    }

    PR_DEBUG("device:%d , channel:%d stream:%d\n",device ,channel,stream);
    max_frame_size = tuya_ipc_ring_buffer_get_max_frame_size(device,channel,stream) ;

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "max_frame_size", max_frame_size);
}

void rpc_ipc_server_tuya_ipc_ring_buffer_sync_stream(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle_video = NULL ;
    RING_BUFFER_USER_HANDLE_T handle_audio = NULL ;
    INT16_T size = 0 ;

    if (OPRT_OK != (ret = jrpc_read_binary(request,JRPC_PARAMS,"handle_video",(uint8_t **)&handle_video,&size))) {
        PR_DEBUG("handle_video err \n");
        goto exit ;
    }

    PR_DEBUG("handle_video:%p\n",handle_video);
    if(handle_video == NULL){
        PR_DEBUG("handle_video err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }

    if (OPRT_OK != (ret = jrpc_read_binary(request,JRPC_PARAMS,"handle_audio",(uint8_t **)&handle_audio,&size))) {
    PR_DEBUG("handle_audio err \n");
    goto exit ;
    }

    PR_DEBUG("handle_audio:%p\n",handle_audio);
    if (handle_audio == NULL){
        PR_DEBUG("handle_audio err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }

    PR_DEBUG("handle:%p,handle_audio:%p\n",handle_video,handle_audio);
    if (OPRT_OK !=  (ret = tuya_ipc_ring_buffer_sync_stream(handle_video,handle_audio))) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("tuya_ipc_ring_buffer_sync_stream err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

}

void rpc_ipc_server_tuya_ipc_ring_buffer_anchor_user(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T handle = NULL ;
    UINT_T frame_num = 0 ;
    BOOL_T check_overlap = 0 ;
    INT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret = jrpc_read_binary(request,JRPC_PARAMS,"handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("handle err \n");
        goto exit ;
    }

    PR_DEBUG("handle:%p\n",tmp_handle);
    if (tmp_handle == NULL) {
        PR_DEBUG("handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    handle = *(UINT64_T*)tmp_handle ;

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"frame_num",&frame_num))) {
        PR_DEBUG("frame_num err \n");
        goto exit ;
    }


    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"check_overlap",&check_overlap))) {
        PR_DEBUG("check_overlap err \n");
        goto exit ;
    }


    PR_DEBUG("handle:%p,frame_num:%d,check_overlap:%d\n",handle,frame_num,check_overlap);
    if (OPRT_OK !=  (ret = tuya_ipc_ring_buffer_anchor_user(handle,frame_num,check_overlap))) {
        ret = OPRT_COM_ERROR ;
        PR_DEBUG("tuya_ipc_ring_buffer_sync_stream err \n");
        goto exit ;
    }

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);

}

void rpc_ipc_server_tuya_ipc_ring_buffer_get_next_gop_encrypted_size(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    RING_BUFFER_USER_HANDLE_T v_handle = NULL ;
    RING_BUFFER_USER_HANDLE_T a_handle = NULL ;
    UINT_T frame_num = 0 ;
    UINT_T encrypted_size = 0 ;
    INT16_T size = 0 ;
    RING_BUFFER_USER_HANDLE_T tmp_handle = NULL ;

    if (OPRT_OK != (ret = jrpc_read_binary(request,JRPC_PARAMS,"v_handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("v_handle err \n");
        goto exit ;
    }

    PR_DEBUG("v_handle:%p\n",tmp_handle);
    if(tmp_handle == NULL) {
        PR_DEBUG("v_handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    v_handle = *(UINT64_T*)tmp_handle ;

    if (OPRT_OK != (ret = jrpc_read_binary(request,JRPC_PARAMS,"a_handle",(uint8_t **)&tmp_handle,&size))) {
        PR_DEBUG("a_handle err \n");
        goto exit ;
    }
    PR_DEBUG("a_handle:%p\n",tmp_handle);
    if (tmp_handle == NULL) {
        PR_DEBUG("a_handle err \n");
        ret = OPRT_COM_ERROR ;
        goto exit ;
    }
    a_handle = *(UINT64_T*)tmp_handle ;

    if (OPRT_OK != (ret =jrpc_read_int(request,JRPC_PARAMS,"frame_num",&frame_num))) {
        PR_DEBUG("frame_num err \n");
        goto exit ;
    }

    encrypted_size = tuya_ipc_ring_buffer_get_next_gop_encrypted_size(v_handle,a_handle,&frame_num);
    
    PR_DEBUG("v_handle:%p,a_handle:%p,frame_num:%d,encrypted_size=%u\n",v_handle,a_handle,frame_num,encrypted_size);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
    jrpc_write_int(reply, JRPC_RESULT, "encrypted_size", encrypted_size);

}

void rpc_ipc_server_tuya_ipc_ring_buffer_adapter_register_media_source(jrpc_msg_t *request, void *arg, jrpc_msg_t *reply)
{
    INT_T ret = OPRT_OK;
    ret = tuya_ipc_ring_buffer_adapter_register_media_source();
    PR_DEBUG("ret = %d\n",ret);

exit:
    jrpc_create_response(reply, 1);
    jrpc_write_int(reply, JRPC_RESULT, "code", ret);
}

void tuya_ipc_rpc_server_declare_ring_buffer()
{
    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_init", rpc_ipc_server_tuya_ipc_ring_buffer_init, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_uninit", rpc_ipc_server_tuya_ipc_ring_buffer_uninit, NULL);
    
    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_set_max_video_afterwards", rpc_ipc_server_tuya_ipc_ring_buffer_set_max_video_afterwards, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_video_num_skill", rpc_ipc_server_tuya_ipc_ring_buffer_get_video_num_skill, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_open", rpc_ipc_server_tuya_ipc_ring_buffer_open, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_close", rpc_ipc_server_tuya_ipc_ring_buffer_close, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_append_data", rpc_ipc_server_tuya_ipc_ring_buffer_append_data, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_append_data_with_timestamp", rpc_ipc_server_tuya_ipc_ring_buffer_append_data_with_timestamp, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_append_extra_data", rpc_ipc_server_tuya_ipc_ring_buffer_append_extra_data, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_frame", rpc_ipc_server_tuya_ipc_ring_buffer_get_frame, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_av_frame", rpc_ipc_server_tuya_ipc_ring_buffer_get_av_frame, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_find_pre_by_frame", rpc_ipc_server_tuya_ipc_ring_buffer_find_pre_by_frame, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_pre_frame", rpc_ipc_server_tuya_ipc_ring_buffer_get_pre_frame, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_next_target_frame", rpc_ipc_server_tuya_ipc_ring_buffer_get_next_target_frame, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_clean_user_state", rpc_ipc_server_tuya_ipc_ring_buffer_clean_user_state, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_clean_user_state_and_buffer", rpc_ipc_server_tuya_ipc_ring_buffer_clean_user_state_and_buffer, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_if_delay", rpc_ipc_server_tuya_ipc_ring_buffer_if_delay, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_max_buf_secs", rpc_ipc_server_tuya_ipc_ring_buffer_get_max_buf_secs, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_set_max_frame_size", rpc_ipc_server_tuya_ipc_ring_buffer_set_max_frame_size, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_max_frame_size", rpc_ipc_server_tuya_ipc_ring_buffer_get_max_frame_size, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_sync_stream", rpc_ipc_server_tuya_ipc_ring_buffer_sync_stream, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_anchor_user", rpc_ipc_server_tuya_ipc_ring_buffer_anchor_user, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_get_next_gop_encrypted_size", rpc_ipc_server_tuya_ipc_ring_buffer_get_next_gop_encrypted_size, NULL);

    tuya_ipc_rpc_server_declare("tuya_ipc_ring_buffer_adapter_register_media_source", rpc_ipc_server_tuya_ipc_ring_buffer_adapter_register_media_source, NULL);
}


