#include <sys/prctl.h>
#include <sys/syscall.h>
#include <pthread.h>
#include <unistd.h>
#include <libgen.h>
#include <signal.h>
#include <fcntl.h>

#include "tuya_iot_config.h"
#include "tuya_ipc_dp_utils.h"
#include "tuya_ipc_common_demo.h"
#include "tuya_ipc_system_control_demo.h"
#include "tuya_ipc_media_demo.h"
#include "tuya_ipc_motion_detect_demo.h"
#include "tuya_ipc_doorbell_demo.h"
#include "tuya_ipc_p2p_demo.h"
#include "tuya_ipc_upgrade_demo.h"
#include "tuya_ipc_low_power_demo.h"
#include "tuya_ipc_log_demo.h"
#include "svc_rpc.h"
#include "tuya_ipc_demo_default_cfg.h"

STATIC INT_T s_mode = -1;
STATIC CHAR_T s_token[30] = {0};

CHAR_T s_raw_path[128] = "/tmp/";

CHAR_T s_ipc_pid[64]="tuya_pid";//Product ID of TUYA device, this is for demo only.
CHAR_T s_ipc_uuid[64]="tuya_uuid";//Unique identification of each device//Contact tuya PM/BD for developing devices or BUY more
CHAR_T s_ipc_authkey[64]="tuya_authkey";//Authentication codes corresponding to UUID, one machine one code, paired with UUID.
CHAR_T s_ipc_storage[64]="/tmp/";//Path to save tuya sdk DB files, should be readable, writeable and storable
CHAR_T s_ipc_env[64]="";
CHAR_T s_ipc_sd_path[128]="/tmp/";//SD card mount directory
CHAR_T s_ipc_upgrade_file[128] = "/tmp/upgrade.file";//File with path to download file during OTA
CHAR_T s_app_version[64] = "1.2.3";//Firemware version displayed on TUYA APP

STATIC TUYA_IPC_SDK_RUN_VAR_S g_sdk_run_info = {0};

void signal_handle(int sig)
{
    char cThreadName[32] = {0};
    prctl(PR_GET_NAME, (unsigned long)cThreadName);
    PR_INFO("[%s, %d] get signal(%d) thread(%ld) name(%s)\n", __FUNCTION__, __LINE__, sig, syscall(__NR_gettid), cThreadName);

    switch(sig) {
    case SIGINT:
    case SIGKILL:
    case SIGTERM:
    case SIGSEGV:
    case SIGABRT:
        exit(1);
        break;

    default:
        break;
    }

    return;
}

VOID IPC_APP_qrcode_shorturl_cb(CHAR_T* shorturl)
{
    if(shorturl)
        PR_INFO("shorturl: %s}", shorturl); //according to requirements of shadow device, do not modify!!!

    return;
}

STATIC VOID * tuya_ipc_sdk_mqtt_online_proc(PVOID_T arg)
{
    PR_DEBUG("tuya_ipc_sdk_mqtt_online_proc thread start success\n");
    while(IPC_APP_Get_MqttStatus() == FALSE)
    {
        sleep(1);
    }
    PR_DEBUG("tuya_ipc_sdk_mqtt_online_proc is start run\n");
    int ret;
    //同步服务器时间
    TIME_T time_utc;
    INT_T time_zone;
    do
    {
        //需要SDK同步到时间后才能开启下面的业务
        ret = tuya_ipc_get_service_time_force(&time_utc, &time_zone);

    } while(ret != OPRT_OK);

    if(FALSE == g_sdk_run_info.quick_start_info.enable)
    {
        IPC_APP_Media_Stream_Init(&(g_sdk_run_info.media_adatper_info));
    }

    if(g_sdk_run_info.event_info.enable)
    {
        ret = tuya_ipc_event_module_init(g_sdk_run_info.event_info.max_event, 
                            g_sdk_run_info.event_info.alarms_of_events, g_sdk_run_info.event_info.alarms_cnt_per_event);
        PR_DEBUG("event module init result is %d\n",ret);
    }

    if(g_sdk_run_info.local_storage_info.enable)
    {
        ret = TUYA_APP_Init_Stream_Storage(&(g_sdk_run_info.local_storage_info));
        PR_DEBUG("local storage init result is %d\n",ret);
    }

    if(g_sdk_run_info.video_msg_info.enable)
    {
        ret =  TUYA_APP_Enable_Video_Msg(&(g_sdk_run_info.video_msg_info));
        PR_DEBUG("door bell init result is %d\n",ret);
    }

    if(g_sdk_run_info.cloud_storage_info.enable)
    {
        ret = TUYA_APP_Enable_CloudStorage(&(g_sdk_run_info.cloud_storage_info));
        PR_DEBUG("cloud storage init result is %d\n",ret);
    }

    IPC_APP_upload_all_status();

    tuya_ipc_upload_skills();
    PR_DEBUG("tuya_ipc_sdk_mqtt_online_proc is end run\n");

    return NULL;
}

STATIC VOID * tuya_ipc_sdk_low_power_p2p_init_proc(VOID * args)
{
    PR_DEBUG("start low power p2p\n");
    //todo process fail
    IPC_APP_Media_Stream_Init(&(g_sdk_run_info.media_adatper_info));
    return NULL;
}

OPERATE_RET tuya_ipc_app_start(IN TUYA_IPC_SDK_RUN_VAR_S * pRunInfo)
{
	if(NULL == pRunInfo)
	{
		PR_ERR("start sdk para is NULL\n");
		return OPRT_INVALID_PARM;
	}

    OPERATE_RET ret = 0;
    STATIC BOOL_T s_ipc_sdk_started = FALSE;
    if(TRUE == s_ipc_sdk_started )
    {
        PR_DEBUG("IPC SDK has started\n");
        return ret;
    }

	memcpy(&g_sdk_run_info, pRunInfo, SIZEOF(TUYA_IPC_SDK_RUN_VAR_S));

	//setup1:init sdk
    TUYA_IPC_ENV_VAR_T env;
    memset(&env, 0, sizeof(TUYA_IPC_ENV_VAR_T));
    strcpy(env.storage_path, pRunInfo->iot_info.cfg_storage_path);
    strcpy(env.product_key,pRunInfo->iot_info.product_key);
    strcpy(env.uuid, pRunInfo->iot_info.uuid);
    strcpy(env.auth_key, pRunInfo->iot_info.auth_key);
    strcpy(env.dev_sw_version, pRunInfo->iot_info.dev_sw_version);
    strcpy(env.dev_serial_num, "tuya_ipc");
    //TODO:raw
    env.dev_raw_dp_cb = pRunInfo->dp_info.raw_dp_cmd_proc;
    env.dev_obj_dp_cb = pRunInfo->dp_info.common_dp_cmd_proc;
    env.dev_dp_query_cb = pRunInfo->dp_info.dp_query;
    env.status_changed_cb = pRunInfo->net_info.ipc_status_change_cb;
    env.upgrade_cb_info.upgrade_cb = pRunInfo->upgrade_info.upgrade_cb;
    env.gw_rst_cb = pRunInfo->iot_info.gw_reset_cb;
    env.gw_restart_cb = pRunInfo->iot_info.gw_restart_cb;
#if defined(QRCODE_ACTIVE_MODE) && (QRCODE_ACTIVE_MODE==1)
    env.qrcode_active_cb = pRunInfo->qrcode_active_cb;
#endif
    env.dev_type = pRunInfo->iot_info.dev_type;
    env.link_type = pRunInfo->net_info.link_type;
    ret = tuya_ipc_init_sdk(&env);
	if(OPRT_OK != ret)
	{
		PR_ERR("init sdk is error\n");
		return ret;
	}

	//设置升级文件路径和名称
	IPC_APP_Upgrade_Set_File(pRunInfo->upgrade_info.upgrade_file);

	//设置日志等级
	tuya_ipc_set_log_attr(pRunInfo->debug_info.log_level,NULL);
	
    //setup2: set media adapter
    IPC_APP_Media_Adapter_Init(&pRunInfo->media_adatper_info, &pRunInfo->media_info);

	//setup 3: ring buffer 创建。
	ret = TUYA_APP_Init_Ring_Buffer(&pRunInfo->media_info.media_info, 0);
	if(OPRT_OK != ret)
	{
		PR_ERR("create ring buffer is error\n");
		return ret;
	}
    tuya_ipc_ring_buffer_adapter_register_media_source();
	//低功耗 优先开启P2P
    if(g_sdk_run_info.quick_start_info.enable)
    {
        pthread_t low_power_p2p_thread_handler;
        int op_ret = pthread_create(&low_power_p2p_thread_handler,NULL, tuya_ipc_sdk_low_power_p2p_init_proc, NULL);
        if(op_ret < 0)
        {
            PR_ERR("create p2p start thread is error\n");
            return -1;
        }
    }

    //创建等待mqtt上线进程，mqtt上线后，再开启与网络相关的业务
    pthread_t mqtt_status_change_handle;
    int op_ret = pthread_create(&mqtt_status_change_handle,NULL, tuya_ipc_sdk_mqtt_online_proc, NULL);
    if(op_ret < 0)
    {
        PR_ERR("create tuya_ipc_sdk_mqtt_online_proc  thread is error\n");
        return -1;
    }
	
	ret = tuya_ipc_start_sdk(pRunInfo->net_info.connect_mode,pRunInfo->debug_info.qrcode_token);
	if(OPRT_OK != ret)
	{
		PR_ERR("start sdk is error\n");
		return ret;
	}

	s_ipc_sdk_started = true;
	PR_DEBUG("tuya ipc sdk start is complete\n");
	return ret;
}

OPERATE_RET TUYA_IPC_SDK_START(WIFI_INIT_MODE_E connect_mode, CHAR_T *p_token)
{
    printf("SDK Version:%s\r\n", tuya_ipc_get_sdk_info());
	TUYA_IPC_SDK_RUN_VAR_S ipc_sdk_run_var ={0};
	memset(&ipc_sdk_run_var,0,sizeof(ipc_sdk_run_var));

	/*certification information(essential)*/
	strcpy(ipc_sdk_run_var.iot_info.product_key,s_ipc_pid);
	strcpy(ipc_sdk_run_var.iot_info.uuid,s_ipc_uuid);
	strcpy(ipc_sdk_run_var.iot_info.auth_key,s_ipc_authkey);
	strcpy(ipc_sdk_run_var.iot_info.dev_sw_version,s_app_version);
	strcpy(ipc_sdk_run_var.iot_info.cfg_storage_path,s_ipc_storage);
	//normal device
	ipc_sdk_run_var.iot_info.dev_type = NORMAL_POWER_DEV;
	//if needed, change to low power device
	//ipc_sdk_run_var.iot_info.dev_type= LOW_POWER_DEV;

	/*connect mode (essential)*/
	ipc_sdk_run_var.net_info.connect_mode = connect_mode;
	ipc_sdk_run_var.net_info.ipc_status_change_cb = IPC_APP_Status_Changed_cb;
    ipc_sdk_run_var.net_info.link_type = TUYA_IPC_LINK_WIFI;
	if(p_token)
	{
	    strcpy(ipc_sdk_run_var.debug_info.qrcode_token,p_token);
	}
	/* 0-5, the bigger, the more log */
	ipc_sdk_run_var.debug_info.log_level = 4;
	/*media info (essential)*/
    /* main stream(HD), video configuration*/
    /* NOTE
    FIRST:If the main stream supports multiple video stream configurations, set each item to the upper limit of the allowed configuration.
    SECOND:E_IPC_STREAM_VIDEO_MAIN must exist.It is the data source of SDK.
    please close the E_IPC_STREAM_VIDEO_SUB for only one stream*/
    ipc_sdk_run_var.media_info.media_info.stream_enable[E_IPC_STREAM_VIDEO_MAIN] = TRUE;    /* Whether to enable local HD video streaming */
    ipc_sdk_run_var.media_info.media_info.video_fps[E_IPC_STREAM_VIDEO_MAIN] = 30;  /* FPS */
    ipc_sdk_run_var.media_info.media_info.video_gop[E_IPC_STREAM_VIDEO_MAIN] = 30;  /* GOP */
    ipc_sdk_run_var.media_info.media_info.video_bitrate[E_IPC_STREAM_VIDEO_MAIN] = TUYA_VIDEO_BITRATE_1M; /* Rate limit */
    ipc_sdk_run_var.media_info.media_info.video_width[E_IPC_STREAM_VIDEO_MAIN] = 640; /* Single frame resolution of width*/
    ipc_sdk_run_var.media_info.media_info.video_height[E_IPC_STREAM_VIDEO_MAIN] = 360;/* Single frame resolution of height */
    ipc_sdk_run_var.media_info.media_info.video_freq[E_IPC_STREAM_VIDEO_MAIN] = 90000; /* Clock frequency */
    ipc_sdk_run_var.media_info.media_info.video_codec[E_IPC_STREAM_VIDEO_MAIN] = TUYA_CODEC_VIDEO_H264; /* Encoding format */

    /* substream(HD), video configuration */
    /* Please note that if the substream supports multiple video stream configurations, please set each item to the upper limit of the allowed configuration. */
    ipc_sdk_run_var.media_info.media_info.stream_enable[E_IPC_STREAM_VIDEO_SUB] = TRUE;     /* Whether to enable local SD video stream */
    ipc_sdk_run_var.media_info.media_info.video_fps[E_IPC_STREAM_VIDEO_SUB] = 30;  /* FPS */
    ipc_sdk_run_var.media_info.media_info.video_gop[E_IPC_STREAM_VIDEO_SUB] = 30;  /* GOP */
    ipc_sdk_run_var.media_info.media_info.video_bitrate[E_IPC_STREAM_VIDEO_SUB] = TUYA_VIDEO_BITRATE_512K; /* Rate limit */
    ipc_sdk_run_var.media_info.media_info.video_width[E_IPC_STREAM_VIDEO_SUB] = 640; /* Single frame resolution of width */
    ipc_sdk_run_var.media_info.media_info.video_height[E_IPC_STREAM_VIDEO_SUB] = 360;/* Single frame resolution of height */
    ipc_sdk_run_var.media_info.media_info.video_freq[E_IPC_STREAM_VIDEO_SUB] = 90000; /* Clock frequency */
    ipc_sdk_run_var.media_info.media_info.video_codec[E_IPC_STREAM_VIDEO_SUB] = TUYA_CODEC_VIDEO_H264; /* Encoding format */

    /* Audio stream configuration.
    Note: The internal P2P preview, cloud storage, and local storage of the SDK are all use E_IPC_STREAM_AUDIO_MAIN data. */
    ipc_sdk_run_var.media_info.media_info.stream_enable[E_IPC_STREAM_AUDIO_MAIN] = TRUE;         /* Whether to enable local sound collection */
    ipc_sdk_run_var.media_info.media_info.audio_codec[E_IPC_STREAM_AUDIO_MAIN] = TUYA_CODEC_AUDIO_PCM;/* Encoding format */
    ipc_sdk_run_var.media_info.media_info.audio_sample [E_IPC_STREAM_AUDIO_MAIN]= TUYA_AUDIO_SAMPLE_8K;/* Sampling Rate */
    ipc_sdk_run_var.media_info.media_info.audio_databits [E_IPC_STREAM_AUDIO_MAIN]= TUYA_AUDIO_DATABITS_16;/* Bit width */
    ipc_sdk_run_var.media_info.media_info.audio_channel[E_IPC_STREAM_AUDIO_MAIN]= TUYA_AUDIO_CHANNEL_MONO;/* channel */
    ipc_sdk_run_var.media_info.media_info.audio_fps[E_IPC_STREAM_AUDIO_MAIN] = 25;/* Fragments per second */

    /*local storage (custome whether enable or not)*/
    ipc_sdk_run_var.local_storage_info.enable = 1;
    ipc_sdk_run_var.local_storage_info.max_event_num_per_day = 500;
    ipc_sdk_run_var.local_storage_info.skills = 0;//0 means all skills
    ipc_sdk_run_var.local_storage_info.sd_status_cb = tuya_ipc_sd_status_upload ;
	strcpy(ipc_sdk_run_var.local_storage_info.storage_path, s_ipc_sd_path);

    /*cloud_storage_info*/
    ipc_sdk_run_var.cloud_storage_info.enable = 1;
    ipc_sdk_run_var.cloud_storage_info.en_audio_record = 1;
    ipc_sdk_run_var.cloud_storage_info.pre_record_time = 2;

	/*media adapter function (essential)*/
	ipc_sdk_run_var.media_adatper_info.max_stream_client = 5;
	ipc_sdk_run_var.media_adatper_info.live_mode = TRANS_DEFAULT_STANDARD;
	ipc_sdk_run_var.media_adatper_info.media_event_cb = __TUYA_APP_p2p_event_cb;
	ipc_sdk_run_var.media_adatper_info.rev_audio_cb = __TUYA_APP_rev_audio_cb;
    ipc_sdk_run_var.media_adatper_info.rev_video_cb = __TUYA_APP_rev_video_cb;
    ipc_sdk_run_var.media_adatper_info.rev_file_cb = __TUYA_APP_rev_file_cb;
    ipc_sdk_run_var.media_adatper_info.get_snapshot_cb = TUYA_APP_get_snapshot_cb;

	/*event function (essential)*/
	TUYA_ALARM_BITMAP_T alarm_of_events[1] = {0};
	tuya_ipc_event_add_alarm_types(&alarm_of_events[0], E_ALARM_MOTION);
	ipc_sdk_run_var.event_info.enable = 1;
	ipc_sdk_run_var.event_info.max_event = sizeof(alarm_of_events) / sizeof(TUYA_ALARM_BITMAP_T);
	ipc_sdk_run_var.event_info.alarms_of_events = alarm_of_events;
	ipc_sdk_run_var.event_info.alarms_cnt_per_event = 1;

	/*AI detect (custome whether enable or not)*/
	ipc_sdk_run_var.cloud_ai_detct_info.enable = 1;

	/*door bell (custome whether enable or not)*/
	ipc_sdk_run_var.video_msg_info.enable = 1;
	ipc_sdk_run_var.video_msg_info.type = MSG_BOTH;
	ipc_sdk_run_var.video_msg_info.msg_duration = 10;

	/*dp function(essential)*/
	ipc_sdk_run_var.dp_info.dp_query = IPC_APP_handle_dp_query_objs;
	ipc_sdk_run_var.dp_info.raw_dp_cmd_proc = IPC_APP_handle_raw_dp_cmd_objs;
	ipc_sdk_run_var.dp_info.common_dp_cmd_proc = IPC_APP_handle_dp_cmd_objs;

	/*upgrade function(essential)*/
	ipc_sdk_run_var.upgrade_info.enable = true;
	ipc_sdk_run_var.upgrade_info.upgrade_cb = IPC_APP_Upgrade_Inform_cb;
	strcpy(ipc_sdk_run_var.upgrade_info.upgrade_file, s_ipc_upgrade_file);

	ipc_sdk_run_var.iot_info.gw_reset_cb = IPC_APP_Reset_System_CB;
	ipc_sdk_run_var.iot_info.gw_restart_cb = IPC_APP_Restart_Process_CB;

    /*QR-active function(essential)*/
#if defined(QRCODE_ACTIVE_MODE) && (QRCODE_ACTIVE_MODE==1)
    ipc_sdk_run_var.qrcode_active_cb = IPC_APP_qrcode_shorturl_cb;
#endif

	OPERATE_RET ret ;
    ret = tuya_ipc_app_start(&ipc_sdk_run_var);
    if(ret !=0 )
    {
    	PR_DEBUG("ipc sdk start fail,please check run parameter，ret=%d\n",ret);
    }
	return ret;
}

static int tuya_ipc_demo_media_cfg_parse(char * cfg_path)
{
    char buffer[4096];
    int read_len = sizeof(buffer);
    int fd = -1;
    ty_cJSON *cfg_json = NULL;

    if (cfg_path == NULL) {
        printf("cfg_path is null\n");
        return -1;
    }
    
    // 从配置路径里读取相关的配置文件，整体的配置环境需要守护程序建立
    fd = open(cfg_path, 2);
    printf("detect auto test enviroment, load config from %s\n", cfg_path);
        
    // 检查是打开文件成功。
    if (fd < 0)
    {
        printf("open path %s failed,%d\n", cfg_path,fd);
        return -1;
    }
        
    // 读取文件内容
    memset(buffer, 0, sizeof(buffer));
    while (read_len == read(fd, buffer, sizeof(buffer)));
    close(fd);
    printf("buffer is %s\n",buffer);
        
    // 解析JSON
    cfg_json = ty_cJSON_Parse(buffer);
    if (NULL == cfg_json)
    {
        printf("load configure failed\n");
        return OPRT_CJSON_PARSE_ERR;
    }

    ty_cJSON* env = ty_cJSON_GetObjectItem(cfg_json, "env");
    if (env) {
        strcpy(s_ipc_env, env->valuestring);
        printf("set env to %s\n", s_ipc_env);
        tuya_svc_mqtt_direct_set_env(s_ipc_env);
    }

    // pid
    ty_cJSON* json_pid = ty_cJSON_GetObjectItem(cfg_json, "pid");
    if (!ty_cJSON_IsString(json_pid)) {
        printf("get pid failed\n");
        ty_cJSON_Delete(cfg_json);
        return -1;
    }
    strcpy(s_ipc_pid, json_pid->valuestring);

    // uuid
    ty_cJSON* json_uuid = ty_cJSON_GetObjectItem(cfg_json, "uuid");
    if (!ty_cJSON_IsString(json_uuid)) {
        printf("get uuid failed\n");
        ty_cJSON_Delete(cfg_json);
        return -1;
    }
    strcpy(s_ipc_uuid, json_uuid->valuestring);

    // authkey
    ty_cJSON* json_authkey = ty_cJSON_GetObjectItem(cfg_json, "authkey");
    if (!ty_cJSON_IsString(json_authkey)) {
        printf("get authkey failed\n");
        ty_cJSON_Delete(cfg_json);
        return -1;
    }
    strcpy(s_ipc_authkey, json_authkey->valuestring);

    // storage path
    ty_cJSON* json_storage = ty_cJSON_GetObjectItem(cfg_json, "storage");
    if (ty_cJSON_IsString(json_storage)) {
        strcpy(s_ipc_storage, json_storage->valuestring);
    }

    // media path
    ty_cJSON* json_media = ty_cJSON_GetObjectItem(cfg_json, "media_path");
    if (ty_cJSON_IsString(json_media)) {
        strcpy(s_raw_path, json_media->valuestring);
    }

    // sd path
    ty_cJSON* json_sd_path = ty_cJSON_GetObjectItem(cfg_json, "sd_path");
    if (ty_cJSON_IsString(json_sd_path)) {
        strcpy(s_ipc_sd_path, json_sd_path->valuestring);
    }

    // upgrade file
    ty_cJSON* json_upgrade_file = ty_cJSON_GetObjectItem(cfg_json, "upgrade_file");
    if (ty_cJSON_IsString(json_upgrade_file)) {
        strcpy(s_ipc_upgrade_file, json_upgrade_file->valuestring);
    }

    // app version
    ty_cJSON* json_app_version = ty_cJSON_GetObjectItem(cfg_json, "app_version");
    if (ty_cJSON_IsString(json_app_version)) {
        strcpy(s_app_version, json_app_version->valuestring);
    }

    // token
    ty_cJSON* json_token = ty_cJSON_GetObjectItem(cfg_json, "token");
    if (ty_cJSON_IsString(json_token)) {
        strcpy(s_token, json_token->valuestring);
    }

    // mode
    ty_cJSON* json_mode = ty_cJSON_GetObjectItem(cfg_json, "mode");
    if (ty_cJSON_IsString(json_mode)) {
        s_mode = atoi(json_mode->valuestring);
    }
    ty_cJSON_Delete(cfg_json);

    printf("s_ipc_pid: %s\n", s_ipc_pid);
    printf("s_ipc_uuid: %s\n", s_ipc_uuid);
    printf("s_ipc_authkey: %s\n", s_ipc_authkey);
    printf("s_ipc_storage: %s\n", s_ipc_storage);
    printf("s_raw_path: %s\n", s_raw_path);
    printf("s_ipc_sd_path: %s\n", s_ipc_sd_path);
    printf("s_ipc_upgrade_file: %s\n", s_ipc_upgrade_file);
    printf("s_app_version: %s\n", s_app_version);
    
    return 0;
}

VOID IPC_APP_simulation()
{
    /* Manual simulation of related functions */
    char test_input[64] = {0};
    while(1)
    {
        scanf("%s",test_input);
        /* Simulation of the start of motion detection events */
        if(0 == strcmp(test_input,"start"))
        {
            IPC_APP_set_motion_status(1);
        }
        /* Simulation of the end of event */
        else if(0 == strcmp(test_input,"stop"))
        {
            IPC_APP_set_motion_status(0);
        }
        /* Simulation of getting device's activation status */
        else if(0 == strcmp(test_input,"status"))
        {
            IPC_REGISTER_STATUS_E status = tuya_ipc_get_register_status();
            PR_INFO("current register status %d[0:unregistered 1:registered 2:activated]\n",status);
        }
        /* Simulation of doorbell press event */
        else if(0 == strcmp(test_input,"bell"))
        {
            //Using demo file for simulation, should be replaced by real snapshot when events happen.
            int snapshot_size = 150*1024;
            char *snapshot_buf = (char *)malloc(snapshot_size);
            int ret = IPC_APP_get_snapshot(snapshot_buf, &snapshot_size);
            if(ret != 0)
            {
                PR_ERR("Get snap fail!\n");
                free(snapshot_buf);
                continue;
            }
            /* Push the detection message and the current snapshot image to the APP.
            Snapshot image acquisition needs to be implemented by the developer */
            tuya_ipc_door_bell_press(DOORBELL_NORMAL, snapshot_buf, snapshot_size, NOTIFICATION_CONTENT_JPEG);
            free(snapshot_buf);
        }
        /* Simulation of low power ipc */
        else if (0 == strcmp(test_input, "start_low_power"))
        {
            tuya_ipc_low_power_sample();
        }
        else if (0 == strcmp(test_input, "ac"))
        {
            doorbell_handler();
        }
        /* Simulation of get time for OSD */
        else if (0 == strcmp(test_input, "osd"))
        {
            IPC_APP_Show_OSD_Time();
        } 
        usleep(100*1000);
    }

    return;
}

VOID usage(CHAR_T *app_name)
{
    PR_INFO("%s -m mode -t token -r raw path -h\n", (CHAR_T *)basename(app_name));
    PR_INFO("\t m: 0-WIFI_INIT_AUTO 1-WIFI_INIT_AP 2-WIFI_INIT_DEBUG, refer to WIFI_INIT_MODE_E\n"
        "\t t: token get form qrcode info\n"
        "\t r: raw source file path\n"
        "\t c: choose sdk run method\n"
        "\t h: help info\n");

    return;
}

extern int tuya_ipc_rpc_server_func_call_cb(ty_cJSON *req_json, ty_cJSON **response);

#ifdef __HuaweiLite__
int app_main(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif

{
    INT_T res = -1;
    CHAR_T token[30] = {0};
    WIFI_INIT_MODE_E mode = WIFI_INIT_AP;

#if defined(WIFI_GW) && (WIFI_GW==0)
    mode = WIFI_INIT_NULL;
#endif

    signal(SIGINT, signal_handle);
    signal(SIGKILL, signal_handle);
    signal(SIGTERM, signal_handle);
    signal(SIGPIPE, SIG_IGN);

    if (0 != tuya_ipc_demo_media_cfg_parse("tuya_device_cfg.json"))
    {
        printf("media cfg parse failed\n");
        return -1;
    }

#ifdef __HuaweiLite__
    if(argc != 2)
    {
        printf("%s <token>\n", argv[0]);
        return -1;
    }
    mode = WIFI_INIT_DEBUG; //The demo mode is set to debug, so before running this main process, 
                            //developers need to make sure that devices are connected to the Internet. 。
    strcpy(token, argv[1]); //Token field values scanned from APP QR-codes or broadcast packets
#else
    while((res = getopt(argc, argv, "?m:t:s:r:p:u:a:h")) != -1)
    {
        switch(res) {
        case 'm':
            mode = atoi(optarg);
            break;
        case 't':
            strcpy(token, optarg);
            break;
        case 'r':
            strcpy(s_raw_path, optarg);
            break;
        case 'p':
            strcpy(s_ipc_pid,optarg);
            break;
        case 'u':
            strcpy(s_ipc_uuid,optarg);
            break;
        case 'a':
            strcpy(s_ipc_authkey,optarg);
            break;
        case 'h':
        default:
            usage(argv[0]);
            return -1;
        }
    }
#endif

    OPERATE_RET ret = OPRT_OK;
    if(s_mode >= 0)
    {
        mode = s_mode;
    }
    if(strlen(s_token))
    {
        strcpy(token, s_token);
    }
    if(mode == WIFI_INIT_AUTO) {
        printf("WIFI_INIT_AUTO is not supported after version 5.6.2\n");
    }
    ret = TUYA_IPC_SDK_START(mode,token);
    if(ret != OPRT_OK)
    {
        return ret;
    }

    int ap_mode_on_off = IPC_APP_get_ap_mode_config();
    if (ap_mode_on_off == 1)
    {
        IPC_APP_switch_ap_mode();
    }

    IPC_APP_Init_Media_Task();
    TUYA_APP_Enable_Motion_Detect();

#ifdef ENABLE_SHADOW_DEVICE
    rpc_set_buffer_size(65535);
    rpc_server_init(tuya_ipc_rpc_server_func_call_cb);
    tuya_ipc_rpc_server_declare_all();
#endif
	
    /* whether SDK is connected to MQTT */
    while(IPC_APP_Get_MqttStatus() != 1)
    {
        usleep(100000);
    }
#ifdef __HuaweiLite__
    tuya_ipc_report_p2p_msg();
#endif

    IPC_APP_simulation();
    return 0;
}

